@@ -47,15 +47,18 @@ enum Gpm_Etype {
   GPM_DOWN=4,
   GPM_UP=  8,
 
-#define GPM_BARE_EVENTS(ev) ((ev)&0xF)
+
+#define GPM_BARE_EVENTS(type) ((type)&(0xF|GPM_ENTER|GPM_LEAVE))
 
   GPM_SINGLE=16,            /* at most one in three is set */
   GPM_DOUBLE=32,
   GPM_TRIPLE=64,            /* WARNING: I depend on the values */
 
   GPM_MFLAG=128,            /* motion during click? */
-  GPM_HARD=256             /* if set in the defaultMask, force an already
+  GPM_HARD=256,             /* if set in the defaultMask, force an already
 			       used event to pass over to another handler */
+  GPM_ENTER=512,            /* enter event, user in Roi's */
+  GPM_LEAVE=1024            /* leave event, used in Roi's */
 };
 
 /*....................................... The event data structure */
