@@ -1,4 +1,4 @@
-/*
+/* -*-mode:C;tab-width:8-*-
  * gpm.h - public include file for gpm-Linux
  *
  * Copyright 1994,1995   rubini@ipvvis.unipv.it (Alessandro Rubini)
@@ -37,6 +37,8 @@
 #define GPM_NODE_PID      GPM_NODE_DIR "/gpmpid"
 #define GPM_NODE_LOG      GPM_NODE_DIR "/gpmlog"
 #define GPM_NODE_CTL      GPM_NODE_DIR "/gpmctl"
+#define GPM_NODE_DEV               "/dev/gpmctl"
+#define GPM_NODE_FIFO              "/dev/gpmdata"
 
 /*....................................... Cfg buttons */
 
@@ -48,11 +50,11 @@
 
 enum Gpm_Etype {
   GPM_MOVE=1,
-  GPM_DRAG=2,   /* exactly one in four is active at a time */
+  GPM_DRAG=2,   /* exactly one of the bare ones is active at a time */
   GPM_DOWN=4,
   GPM_UP=  8,
 
-#define GPM_BARE_EVENTS(ev) ((ev)&0xF)
+#define GPM_BARE_EVENTS(type) ((type)&(0x0f|GPM_ENTER|GPM_LEAVE))
 
   GPM_SINGLE=16,            /* at most one in three is set */
   GPM_DOUBLE=32,
@@ -60,7 +62,7 @@ enum Gpm_Etype {
 
   GPM_MFLAG=128,            /* motion during click? */
   GPM_HARD=256,             /* if set in the defaultMask, force an already
-			       used event to pass over to another handler */
+                   used event to pass over to another handler */
 
   GPM_ENTER=512,            /* enter event, user in Roi's */
   GPM_LEAVE=1024            /* leave event, used in Roi's */
@@ -111,7 +113,7 @@ typedef struct Gpm_Roi {
   unsigned short eventMask;
   unsigned short owned;
   Gpm_Handler *handler;
-  void *xtradata;
+  void *clientdata;
   struct Gpm_Roi *prev;
   struct Gpm_Roi *next;
 }              Gpm_Roi;
@@ -121,9 +123,6 @@ typedef struct Gpm_Roi {
 
 extern int gpm_flag, gpm_ctlfd, gpm_fd, gpm_hflag, gpm_morekeys;
 
-extern Gpm_Handler *gpm_handler;
-extern void *gpm_data;
-
 extern int gpm_zerobased;
 extern int gpm_visiblepointer;
 extern int gpm_mx, gpm_my; /* max x and y to fit margins */
@@ -132,8 +131,15 @@ extern struct timeval gpm_timeout;
 extern unsigned char    _gpm_buf[];
 extern unsigned short * _gpm_arg;
 
-/* this can be used to access the high-level library */
+extern Gpm_Handler *gpm_handler;
+extern void *gpm_data;
+
+extern Gpm_Handler *gpm_roi_handler;
+extern void *gpm_roi_data;
+
 extern Gpm_Roi *gpm_roi;
+extern Gpm_Roi *gpm_current_roi;
+ 
 
 /*....................................... Prototypes for the client       */
 /*                                          all of them return 0 or errno */
@@ -149,13 +155,20 @@ extern int Gpm_Getc(FILE *);
 #define    Gpm_Getchar() Gpm_Getc(stdin)
 extern int Gpm_Repeat(int millisec);
 extern int Gpm_FitValuesM(int *x, int *y, int margin);
-#define    Gpm_FitValues(x,y) Gpm_FitValuesM((int *)(x),(int *)(y),-1);
-#define    Gpm_FitEvent(ePtr) \
-                        ((ePtr)->margin \
-                         ? Gpm_FitValuesM((int *)(&((ePtr)->x)), \
-					  (int *)(&((ePtr)->y)), \
-                                (ePtr)->margin) \
-                         : 0)
+#define    Gpm_FitValues(x,y) Gpm_FitValuesM((x),(y),-1);
+#define    Gpm_FitEvent(ePtr)   \
+   do {                         \
+      int _x, _y;               \
+      if ((ePtr)->margin)       \
+        {                       \
+        _x = (ePtr)->x;         \
+        _y = (ePtr)->y;         \
+        Gpm_FitValuesM(&_x, &_y, (ePtr)->margin); \
+        (ePtr)->x = _x;         \
+        (ePtr)->y = _y;         \
+	}                       \
+    } while(0)                  \
+
 
 /* the following is a (progn ...) form */
 
@@ -183,9 +196,10 @@ extern int gpm_consolefd; /* liblow.c */
 
 
 /* libhigh.c */
-int Gpm_HandleRoi(Gpm_Event *ePtr);
+
+Gpm_Handler Gpm_HandleRoi;
 Gpm_Roi *Gpm_PushRoi(int x, int y, int X, int Y, int mask,
-					 Gpm_Handler *fun, void *xtradata);
+                     Gpm_Handler *fun, void *xtradata);
 Gpm_Roi * Gpm_PopRoi(Gpm_Roi *which);
 Gpm_Roi * Gpm_RaiseRoi(Gpm_Roi *which, Gpm_Roi *before);
 Gpm_Roi * Gpm_LowerRoi(Gpm_Roi *which, Gpm_Roi *after);
@@ -197,6 +211,11 @@ Gpm_Roi * Gpm_LowerRoi(Gpm_Roi *which, Gpm_Roi *after);
 extern int Gpm_Wgetch();
 #define Gpm_Getch() (Gpm_Wgetch(NULL))
 
+/* libxtra.c */
+char *Gpm_GetLibVersion(int *where);
+char *Gpm_GetServerVersion(int *where);
+int   Gpm_GetSnapshot(Gpm_Event *ePtr);
+
 #endif /* _GPM_H_ */
 
 
